function [holdphi, holdsigmasqv, holdmean, holdvar, holdM, holdsigmasq, holdlogsigmasq, predstar,holdk,holdW, TIME] = fitmxzerosv2(data, a, Mprior, sigmasq0, numbofits, burnin, every, c)
%%% FITMXZEROSV2: Code for SV-SPM of ``Bayesian Nonparametric Modelling of the
%%% Return Distribution with Stochastic Volatility''
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%% Input Arguments
%%% data:      log(y^2+10^{-c}),
%%% a:         smoothing parameter either 0.01 or 0.05,
%%% Mprior:    prior for parameter M,
%%% sigmasq0:  variance for the zero component,
%%% numbofits: number of iterations,
%%% burnin:    number of iterations that consist of the burn-in period,
%%% every:     thinning parameter,
%%% c:         offset parameter.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%% Ouput Arguments
%%% holdphi:        draws of the persistence parameter $\phi$,
%%% holdsigmasqv:   draws of the variace of log-volatility
%%%                 $\sigma^{2}_{\eta}$,
%%% holdmean:       draws of the weighted mean of the DPM,
%%% holdvar:        draws of the weighted variance of the DPM,
%%% holdM:          draws of the parameter $M$,
%%% holdsigmasq:    draws of the variance $\sigma^{2}_{z}$,
%%% holdlogsigmasq: draws of the log-volatility,
%%% predstar:       draws of the predictive density,
%%% holdk:          draws of number of components $k$,
%%% holdW:          draws of the weight of allocation to the zero
%%%                 component $W$,
%%% TIME:           time of the programme to run.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

tic

datatrans = log(10.^(-c)+data.^2);%%%% transforming the data

mufi=0;sigmafi=10; %%% prior for phi
phi=truncatednormal(1,mufi,sigmafi,-1,1);

locsigma=2.5;scalesigma=0.01*locsigma;%%% prior for sigmasqv
sigmasqv=1./(gamrnd(locsigma,1,1)./(scalesigma));

n = length(datatrans);

sigmasq = 1;%%% initial value for sigmasq

M = 3;
Maccept = 0;
Mcount = 0;

mu0 = 0; %%% initial value for mu0
emean = 0; %%% readjusted mean

s = zeros(1,n); %%% indicator variables

W = dirichlet([0.1 0.9]); %%% weights for zero comp and CCV

for j=1:n, s(j) = discrete(W);end
n1=sum(s==1); n2=sum(s~=1);
mu = [log(10^(-c)) 0];
numb = [sum(s==1) sum(s==2)];

holdphi = zeros(1, numbofits);
holdsigmasqv = zeros(1, numbofits);
holdsigmasq = zeros(1, numbofits);
holdlogsigmasq = zeros(numbofits,n);
holdM=zeros(1,numbofits);
holdvar=zeros(1,numbofits);
holdmean=zeros(1,numbofits);
holdk=zeros(1,numbofits);
holdW=zeros(1,numbofits);
x = linspace(-20, 5, 1000); %%% predictive
predstar = zeros(1, length(x));

probparam=[0.5 0.5]; %%% probabilities for alternative parametrization case 1: non-centered case 2: centered

for it = 1:(burnin+every*numbofits)

    cases=discrete(probparam);
    %%% update volatilities
    sigmasqvol = [sigmasq0 ones(1,length(mu)-1).*a.*sigmasq];
    logsigmasq= ckfilternew(1,datatrans',1,phi,sqrt(sigmasqv),0,s,mu,sqrt(sigmasqvol));
    
    %%% update indicators
    for i = 1 : n
        numb(s(i)) = numb(s(i)) - 1;
        if ( numb(s(i)) == 0 && s(i)~=1 )
            numb = [numb(1:(s(i)-1)) numb((s(i)+1):length(numb))];
            mu = [mu(1:(s(i)-1)) mu((s(i)+1):length(mu))];
            s = s - (s > s(i));
        end
        logprob = zeros(1, length(mu) + 1);
        logprob(1) = log (W(1)) - 0.5 * log(sigmasq0) - 0.5 * (datatrans(i) - logsigmasq(i) - mu(1)).^2 ./ sigmasq0;
        logprob(2:length(mu)) = log(W(2)) + log( numb(2:end) ) - log(n2-1+M) -   0.5 .* (datatrans(i) - logsigmasq(i) - mu(2:end)).^2 ./ (a * sigmasq) - 0.5 * log(a * sigmasq);
        logprob(length(logprob)) = log(W(2)) + log(M) - log(n2-1+M) - 0.5 .* (datatrans(i) - logsigmasq(i) - mu0).^2 / sigmasq - 0.5.*log(sigmasq);

        prob = exp(logprob - max(logprob));

        fprob = cumsum(prob);
        fprob = fprob / fprob(end);

        u = rand;
        s(i) = 1;
        while ( fprob(s(i)) < u )
            s(i) = s(i) + 1;
        end
        n1=sum(s==1); n2=sum(s~=1);



        if ( s(i) > length(mu)  )
            var1 = 1/(1 / (a * sigmasq) + 1 / ((1 - a) * sigmasq));
            mean1 = var1 * ((datatrans(i)-logsigmasq(i)) / (a * sigmasq) + mu0 / ((1 - a) * sigmasq));
            newmu = mean1 + sqrt(var1) * randn;
            mu = [mu newmu];
            numb = [numb 1];
        else
            numb(s(i)) = numb(s(i)) + 1;
        end
    end
    %%% update mixing components
    for i = 2:length(mu)
        var1 = 1 / (sum(s==i) / (a*sigmasq) + 1 / ((1-a)*sigmasq));
        mean1 = var1 * (sum((datatrans(s==i) - logsigmasq(s==i)) / (a*sigmasq)) + mu0 / ((1 - a) * sigmasq));

        mu(i) = mean1 + sqrt(var1) * randn;
    end


    W = dirichlet([n1+0.1 n2+0.9]);
    %%% Update adjusted mean
    emean=W(2).*(sum(numb(2:end).*mu(2:end))./(n2+M)+M.*mu0./(n2+M)) +W(1).*log(10^(-c))+1.2704;
    %%% Update variance of Dirichlet
    vardir=W(2).*(sum(numb(2:end).*(a.*sigmasq+mu(2:end).^2))+M*(mu0.^2+sigmasq))./(n2+M)-(emean-1.2704)^2+W(1).*(log(10^(-c)).^2+1);
    %%% Update M
    newM = M * exp(randn);

    logaccept = gammaln(newM) - gammaln(newM + n2) + (length(mu)-1) * log(newM);
    logaccept = logaccept - gammaln(M) + gammaln(M + n2) - (length(mu)-1) * log(M);
    logaccept = logaccept + Mprior(1) * (log(newM) - log(M)) - 2 * Mprior(1) * (log(newM + Mprior(2)) - log(M + Mprior(2)));

    accept = 1;
    if ( logaccept < 0 )
        accept = exp(logaccept);
    end

    if ( rand < accept )
        M = newM;
    end

    Maccept = Maccept + accept;
    Mcount = Mcount + 1;
    if cases==1 %%%non-centered
        %% Update mu0
        mean1 = mean(mu(2:end));
        var1 = (1 - a) * sigmasq / length(mu(2:end));
        mu0 = mean1 + sqrt(var1) * randn;
        %% Update sigmasq
        loc = find(s~=1);
        alphastar = 0.5 * ( length(loc) + length(numb(2:end)));
        betastar = 0.5 * (sum((datatrans(loc)-logsigmasq(loc) - mu(s(loc))).^2) / a + sum((mu(2:end) - mu0).^2) / (1 - a));
        sigmasq = 1 / gamrnd(alphastar, 1 / betastar);
        %% Update phi
        mean1 = sum(logsigmasq(1:(n-1)).*logsigmasq(2:n)) / sum(logsigmasq(1:(n-1)).^2);
        var1 = sigmasqv / sum(logsigmasq(1:(n-1)).^2);

        phistar = truncatednormal(1,mean1,sqrt(var1),-1,1);
        gfstar=gf1(logsigmasq(1),mufi,sigmafi,phistar,sigmasqv);
        gfprevious=gf1(logsigmasq(1),mufi,sigmafi,phi,sigmasqv);
        p=exp(gfstar-gfprevious);
        if rand<p
            phi=phistar;
        end
        %% Update sigmasqv
        locp=locsigma+n/2;
        sumscalep=sum((logsigmasq(2:n)-phi*logsigmasq(1:(n-1))).^2);
        scalep=scalesigma+sumscalep/2+(logsigmasq(1)^2*(1-phi^2))/2;

        sigmasqv=1./(gamrnd(locp,1,1)./scalep);

    else

        %%% change parametrization to centered
        logsigmasqstar=logsigmasq+mu0;
        mustar=mu-mu0;
        %% Update mu0
        var1=sigmasqv*((n-1)*(1-phi)^2+(1-phi^2))^(-1);
        mean1=var1*((1-phi^2)/sigmasqv*logsigmasqstar(1)+(1-phi)/sigmasqv*sum((logsigmasqstar(2:end)-phi.*logsigmasqstar(1:end-1))));
        mu0=normrnd(mean1,sqrt(var1));
        %% Update sigmasq
        loc = find(s~=1);
        alphastar = 0.5 * (length(loc) + length(numb(2:end)));
        betastar = 0.5 * (sum((datatrans(loc)-logsigmasqstar(loc) - mustar(s(loc))).^2) / a + sum((mustar(2:end) - 0).^2) / (1 - a));

        sigmasq = 1 / gamrnd(alphastar, 1 / betastar);
        %% Update phi
        mean1=sum((logsigmasqstar(2:n)-mu0).*(logsigmasqstar(1:(n-1))-mu0))./sum((logsigmasqstar(1:(n-1))-mu0).^2);
        var1=sigmasqv*sum((logsigmasqstar(1:(n-1))-mu0).^2)^(-1);
        phistar=truncatednormal(1,mean1,sqrt(var1),-1,1);
        gfstar=cgf1(logsigmasqstar(1),mufi,sigmafi,mu0,phistar,sigmasqv);
        gfprevious=cgf1(logsigmasqstar(1),mufi,sigmafi,mu0,phi,sigmasqv);
        p=exp(gfstar-gfprevious);
        if rand<p
            phi=phistar;
        end
        %% Update sigmasqv
        locp=locsigma+n/2;

        sumscalep=sum((logsigmasqstar(2:n)-mu0-phi.*(logsigmasqstar(1:(end-1))-mu0)).^2);
        scalep=scalesigma+sumscalep/2+((logsigmasqstar(1)-mu0)^2*(1-phi^2))/2;
        sigmasqv=1./(gamrnd(locp,1,1)./scalep);
        %% change parametrization to non-centered
        logsigmasq= logsigmasqstar-mu0;
        mu=mustar+mu0;
    end
    if (mod(it,100)==0)
        disp((it - burnin) / every)
        disp([phi sigmasqv emean])
        disp([length(mu)-1 n1])
    end
    
    %% Predictive Density
    if ( (it > burnin) && (mod(it - burnin, every) == 0) )

        for i = 1:length(x)
            predstar(i) = predstar(i) + W(2).*sum(numb(2:end)./(n2+M).*exp(-0.5 * (x(i) - mu(2:end)).^2 ./ (a * sigmasq)) / sqrt(2*pi*a * sigmasq));
            predstar(i) = predstar(i) + W(2).*M / (n2 + M) * exp(-0.5 * (x(i) - mu0)^2 / sigmasq) / sqrt(2*pi*sigmasq);
            predstar(i) = predstar(i) + W(1).* normpdf(x(i), log(10^(-c)), sqrt(sigmasq0));
        end
        
        logsigmasqS= logsigmasq;
        for j= loc, logsigmasqS(j) = logsigmasqS(j) + emean; end

        holdphi((it - burnin) / every) = phi;
        holdsigmasqv((it - burnin) / every) = sigmasqv;
        holdsigmasq((it - burnin) / every) = sigmasq;

        holdlogsigmasq((it - burnin) / every,:)=logsigmasqS;

        holdM((it - burnin) / every)=M;
        holdk((it - burnin) / every)=length(mu)-1;
        holdW((it - burnin) / every)=W(1);
        holdvar((it - burnin) / every)=vardir;
        holdmean((it - burnin) / every)=emean;
    end
end

predstar = predstar / numbofits;
TIME = toc;

%%%%%%%%%%%%%%%%%%% Display Results %%%%%%%%%%%%%%%%%%%%%%%%
disp(['The posterior mean of'])
disp(['\phi is=' num2str(mean(holdphi))])
disp(['\sigma^{2}_{\eta} is=' num2str(mean(holdsigmasqv))])
disp(['\mu is=' num2str(mean(holdmean))])



end

